//
//les includes
//
#include "libarp.h"
#include <pthread.h>
#include <time.h>

//
//definitions
//
#define VERSION 1.0

//
//les structures
//
typedef struct arp_record
{
   char ip[17];
   char mac[18];
} arp_field;

struct arp_map
{
   char *device;
   int tab_length;
   char requested_ip[17]; 
   arp_field arp_tab[50];
};

//
//les prototypes
//
void *arplisten(void *data);
int ifvrfy(char *intf);

//
//programme principal
//
int main(int argc, char **argv)
{
   //les variables
   int i, c, cache_query, comp;
   char *my_hw_addr;
   char *my_ip_addr;
   char *targ_hw_addr;
   char cache_hw_addr[18], rep;
   int sending_socket;
   int receiving_socket;
   struct arp_packet *request_pkt;
   struct arp_packet_feature *pkt_feat;
   struct ether_addr *arplookup;
   struct arp_map map;
   pthread_t arp_listen;
   unsigned long u;
   char *arg_command, *arg_device, *arg_target;
   char **parse_command;
   int mode_single, mode_command;
   extern char *optarg;
   
   
   //traitement des arguments de ligne de commande
   //
   mode_single = 0;
   mode_command = 0;
   arg_device = "eth0";
   arg_target = NULL;
   while((c = getopt(argc, argv, "sc:d:t:")) != -1)
   {
      switch(c)
      {
         case 's':
	           mode_single = 1;
	           break;
	 case 'c':
	           mode_command = 1;
	           arg_command = optarg;
	           break;
	 case 'd':
	           arg_device = optarg;
	           break;
	 case 't':
	           arg_target = optarg;
	           break;
	 default:
	 	  usage();
      }
   }  
   if (mode_single == 1 && mode_command == 1) usage();
   if (arg_target == NULL) usage();
   if ( ifvrfy(arg_device) != 0)
   {
      printf("interface %s do not exist\n",arg_device);
      usage(); 
   }
   
   
   //Lecture de la table ARP du systme local
   //
   arplookup = (struct ether_addr *)malloc(sizeof(struct ether_addr));
   memcpy(&u, request_pkt->rcpt_ip_addr, 4); 
   cache_query = arp_cache_lookup(arg_device, u, arplookup);
   strncpy(cache_hw_addr, get_hw_addr(arplookup->ether_addr_octet),18);
   
   
   //initialisation des paramtres ncessaires qu thread d'ecoute
   //
   map.device = (char*)malloc(sizeof(arg_device));
   strncpy(map.device, arg_device, sizeof(arg_device));
   strncpy(map.requested_ip, arg_target, sizeof(map.requested_ip));
   
     
   //Envoi d'une requte ARP pour l'IP cible et dclenchement du thread d'ecoute 
   //
   pkt_feat = (struct arp_packet_feature*)malloc(sizeof(struct arp_packet_feature));
   request_pkt = (struct arp_packet*)malloc(sizeof(struct arp_packet));
   pkt_feat->op_code = OP_ARP_REQUEST;
   strncpy(pkt_feat->requested_ip,arg_target, sizeof(pkt_feat->requested_ip));
   init_arp_packet(request_pkt, pkt_feat, arg_device);
   sending_socket = create_arp_send_socket();
   pthread_create(&arp_listen, NULL, arplisten, (void *) &map);
   for (i=0; i<3; i++)
   {
      send_arp_packet(sending_socket, arg_device, request_pkt);
   }
   close(sending_socket);
   
   
   
   
   //Recherche d'un spoof ARP et dclenchement d'alerte
   //
   pthread_join(arp_listen, NULL);
   if ( map.tab_length == 0 )
   {
      rep = 'y';
      printf("No ARP REPLY. Continue([y]/n)?\n");
      rep = getchar();
      if (rep != 'y') exit(1); 
   }
   else
   {
      i=1;
      
      while (i < map.tab_length)
      {
         comp = strncmp(map.arp_tab[i].mac, map.arp_tab[0].mac, 18);
         if ( comp != 0 )
	 {
             printf("receiving multiple ARP reply: SPOOF detected !!!\n");
	     printf("dangerous MAC->%s, %s\n",map.arp_tab[i].mac,map.arp_tab[0].mac);
	     //break;
             exit(0); 
         }
         i++; 
      }
      comp = strncmp(cache_hw_addr, map.arp_tab[0].mac, 18);
      if ( comp != 0 && cache_query == 0)
      {
        printf("incoherent ARP cache status: SPOOF detected !!!\n");
	    printf("dangerous MAC->%s, %s\n",cache_hw_addr,map.arp_tab[0].mac);
        exit(0);
      }
   }
   
   
   //Execution de la commande utilisateur
   //   
   if (mode_command == 1)
   {
      parse_command = (char**)malloc(sizeof(char*));
      parse_command[0] = strtok(arg_command, " ");
      i = 0;
      while(parse_command[i] != NULL)
      {
         parse_command[i+1] = strtok(NULL, " ");
	 i++;
      }
      execvp(parse_command[0], parse_command);
   }
}

//
//implantation des fonctions
//

//fonction dclenche dans le thread (ecoute et stockage des reponses ARP)
//
void *arplisten(void *data)
{
   struct arp_packet *pkt;
   int sock,eq,i;
   time_t tempo, max;
   struct arp_map *arptab =(struct arp_map*)data;
   
   tempo = time(NULL);   
   max = tempo + 1;
   i = 0;
   sock = create_arp_receive_socket(arptab->device);
   
   while (tempo < max)
   {
       recv_arp_packet(sock, &pkt);
       tempo = time(NULL);
       
       if ( ntohs(pkt->op) == OP_ARP_REPLY )
       {
          eq = strncmp(get_ip_addr(pkt->sndr_ip_addr), arptab->requested_ip,sizeof(arptab->requested_ip));
	  if(eq == 0 && i<50)
	  {
	     //printf("%s is at %s\n",get_ip_addr(pkt->sndr_ip_addr), get_hw_addr(pkt->sndr_hw_addr));
	     strncpy(arptab->arp_tab[i].ip, get_ip_addr(pkt->sndr_ip_addr), sizeof(arptab->arp_tab[i].ip));
	     strncpy(arptab->arp_tab[i].mac, get_hw_addr(pkt->sndr_hw_addr), sizeof(arptab->arp_tab[i].mac));
	     i++;
	  }
       }
   }
   arptab->tab_length = i;
   close(sock);
}



//fonction affichant les paramtres de ligne de commande
//
usage(void)
{
        fprintf(stderr,"Version: 1.0\n"
                "Usage: arpwrap -s [-d interface] -t target\n"
		"       arpwrap -c \"command\" [-d interface] -t target\n"
		"\n"
		"arpwrap has two modes: single (-s) and command (-c).\n"
		"Use the single mode to test an IP you suspect to be spoofed.\n"
		"Use the command mode to secure a command execution.\n"
		"If the command's target is an external host, you must\n"
		"indicate your gateway as arpwrap target.\n\n");
        exit(1);
}


//fonction retournant l'ensemble des interfaces rseau
//
int ifvrfy(char *intf)
{
   int fd, i, comp;
   char buff[1024];
   struct ifconf ifc;
   struct ifreq *ifr;
   
   ifc.ifc_len = sizeof(buff);
   ifc.ifc_buf = buff;
   fd = socket(AF_INET, SOCK_DGRAM, 0);
   if (ioctl(fd, SIOCGIFCONF, &ifc) < 0) 
   {
      fprintf(stderr, "SIOCGIFCONF: %s\n", strerror(errno));
      return;
   }
   close(fd);
   
   ifr = ifc.ifc_req;
   comp = -1;
   for (i = ifc.ifc_len / sizeof(struct ifreq); --i >= 0; ifr++) 
   {     
      comp = strncmp(ifr->ifr_name, intf, sizeof(ifr->ifr_name));
      if (comp == 0)
      {
         return(comp);
      }  
   }
   return(comp);
}


//fonction vrifiant le formatage de l'adresse cible
//
char* ipvrfy(char *ip)
{

}
