//
//les inclusions
//
#include "libarp.h"


//
//les fonctions
//

//Cration d'une raw-socket capable d'encapsuler
//un paquet ARP. 
//
int create_arp_send_socket()
{
   int sock;
   
   if ( (sock = socket(AF_INET, SOCK_PACKET, htons(ETH_P_ARP))) < 0)
   {
      die("impossible d'ouvrir la socket\n");
   }
   else return(sock);
}




int create_arp_receive_socket(char* dev)
{
   int sock;
   struct sockaddr sa;
   
   if ( (sock = socket(AF_INET, SOCK_PACKET, htons(ETH_P_ARP))) < 0)
   {
      die("impossible d'ouvrir la socket\n");
   }
   
   memset(&sa, 0, sizeof(sa));
   sa.sa_family = AF_INET;
   strncpy(sa.sa_data, dev, sizeof(sa.sa_data));
   if ( bind(sock, &sa, sizeof(sa)) < 0 )
   {
      close(sock);
      die("create_arp_receive_socket: binding failed\n");
   }
   return (sock);
}





//fabrication d'un paquet ARP  partir de spcifications
//donnes par l'utilisateur.
//
void init_arp_packet(struct arp_packet* pkt, struct arp_packet_feature* feat, char* dev)
{
   char local_mac[18];
   char local_ip[18];
   struct in_addr src_in_addr;
   struct in_addr targ_in_addr;
   
   get_local_hw_addr(dev, local_mac);
   get_local_ip_addr(dev, local_ip);

   if(feat->op_code == 1 || feat->op_code == 2) pkt->op = htons(feat->op_code);
   else die("bad arp code operation\n");
   
   pkt->frame_type = htons(ARP_FRAME_TYPE); 
   pkt->hw_type = htons(ETHER_HW_TYPE);
   pkt->prot_type = htons(IP_PROTO_TYPE);
   pkt->hw_addr_size = ETH_HW_ADDR_LEN;
   pkt->prot_addr_size = IP_ADDR_LEN;
   bzero(pkt->padding, 18);

   if(feat->op_code == OP_ARP_REQUEST)
   {
      put_hw_addr(pkt->targ_hw_addr, "ff:ff:ff:ff:ff:ff");
      put_hw_addr(pkt->src_hw_addr, local_mac);
      put_hw_addr(pkt->rcpt_hw_addr, "00:00:00:00:00:00");
      put_hw_addr(pkt->sndr_hw_addr,local_mac);
      put_ip_addr(&src_in_addr, local_ip);
      put_ip_addr(&targ_in_addr, feat->requested_ip);
      memcpy(pkt->sndr_ip_addr, &src_in_addr, IP_ADDR_LEN);
      memcpy(pkt->rcpt_ip_addr, &targ_in_addr, IP_ADDR_LEN);
   }
   else
   {
      put_hw_addr(pkt->targ_hw_addr, feat->replied_mac);
      put_hw_addr(pkt->src_hw_addr, feat->mac_info);
      put_hw_addr(pkt->rcpt_hw_addr, feat->replied_mac);
      put_hw_addr(pkt->sndr_hw_addr,feat->mac_info);
      put_ip_addr(&src_in_addr, feat->ip_info);
      put_ip_addr(&targ_in_addr,feat->replied_ip);
      memcpy(pkt->sndr_ip_addr, &src_in_addr, IP_ADDR_LEN);
      memcpy(pkt->rcpt_ip_addr, &targ_in_addr, IP_ADDR_LEN);
   }   
}





//envoi d'un paquet ARP
//
void send_arp_packet(int sock, char* dev, struct arp_packet* pkt)
{
   struct sockaddr sa;
   
   memset(&sa, 0, sizeof(sa));
   sa.sa_family = AF_INET;
   strncpy(sa.sa_data, dev, sizeof(sa.sa_data));
   if (sendto(sock, pkt, sizeof(*pkt), 0, &sa, sizeof(sa)) < 0)
   {
      die("arp packet sending error\n");
   }
}




//rception d'un paquet ARP
//
int recv_arp_packet(int sock, struct arp_packet** pkt)
{
   void *buffer;
   int pkt_length,i;
   u_char* buf;
   buffer = (void *)malloc(64); 
   pkt_length = recvfrom(sock, buffer, 64, 0, NULL, NULL);
   if (pkt_length <= 0)
   {
      die("arp packet receive error\n");
   }
     
   *pkt = (struct arp_packet*) buffer;
   return(pkt_length);   
}




//conversion d'une adresse mac sous forme de chaine de
//caractres en mac sous forme de bits.
//
void put_hw_addr(u_char* rcpt, char* mac)
{
      int i;
      char c,val = 0;

   for(i=0;i<ETH_HW_ADDR_LEN;i++)
   {
      if( !(c = tolower((unsigned int)(*mac++)))) die("Invalid hardware address");
      if(isdigit((unsigned int)c)) val = c-'0';
      else if(c >= 'a' && c <= 'f') val = c-'a'+10;
      else die("Invalid hardware address");

      *rcpt = val << 4;
      if( !(c = tolower(*mac++))) die("Invalid hardware address");
      if(isdigit((unsigned int)c)) val = c-'0';
      else if(c >= 'a' && c <= 'f') val = c-'a'+10;
      else die("Invalid hardware address");

      *rcpt++ |= val;

      if(*mac == ':')mac++;
    }
}




//Conversion d'unr adresse mac sous forme de bits
//en chaine de caracteres.
//
char* get_hw_addr(u_char* rcpt)
{
  int i,j;
   char *str;
   char hexatab[16]={'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};
   j=0;
   str = (char *)malloc(18);
   
   for (i=0; i<ETH_HW_ADDR_LEN; i++)
   {
     str[j]=hexatab[rcpt[i] >> 4];
     str[j+1]=hexatab[rcpt[i]&0x0F];
     if (i < 5) str[j+2]=':';
     j = j + 3;
   }
   str[17] = '\0';
   return(str);
}




//conversion d'une ip sous forme de chaine de caractres
//en ip sous forme de bits.
//
void put_ip_addr(struct in_addr* rcpt, char* ip)
{
   struct hostent *hostp;

   rcpt->s_addr=inet_addr(ip);
   if(rcpt->s_addr == -1)
   {
      if( (hostp = gethostbyname(ip)))
      {
         memcpy(rcpt, hostp->h_addr, hostp->h_length);
      }
      else
      {
         printf("put_ip_addr: unknown host %s\n",ip);
	 die("");
      }
   }
}




//conversion d'une ip sous formes de bits
//en ip sous forme de chaine de caractres.
//
char* get_ip_addr(u_char* ip)
{
   struct in_addr addr;
   memcpy(&addr, ip, sizeof(struct in_addr));
   return ( inet_ntoa(addr) );
} 




//fonction renvoyant l'adresse MAC de la carte indique en paramtre
//
void get_local_hw_addr(char* dev, char* mac)
{
    int sock;
    struct ifreq ifr;
    u_char *str;
    
    if ( (sock = socket(AF_INET, SOCK_PACKET, htons(ETH_P_ALL))) < 0 )
    {
       die("get_local_ip_addr: ouverture de socket impossible\n");
    }
    
    memset(&ifr, 0, sizeof(ifr));
    strncpy(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
    if( ioctl(sock, SIOCGIFHWADDR, &ifr) < 0 )
    {
       die("get_local_mac_addr: erreur lecture adresse mac\n");
    }
    close(sock);
    str = (u_char *) ifr.ifr_hwaddr.sa_data;
    sprintf(mac, "%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x\n", str[0], str[1], str[2], str[3], str[4], str[5]);
}



//fonction renvoyant l'adresse IP de la carte indique en paramtre
//
void get_local_ip_addr(char* dev, char* ip)
{
    int sock;
    struct ifreq ifr;
    u_char *str;
    
    if ( (sock = socket(AF_INET, SOCK_PACKET, htons(ETH_P_ALL))) < 0 )
    {
       die("get_local_ip_addr: ouverture de socket impossible\n");
    }
    
    memset(&ifr, 0, sizeof(ifr));
    strncpy(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
    if( ioctl(sock, SIOCGIFADDR, &ifr) < 0 )
    {
       die("get_local_ip: erreur lecture adresse IP\n");
    }
    close(sock);
    str = (u_char *) ifr.ifr_addr.sa_data;
    sprintf(ip, "%d.%d.%d.%d\n", str[2], str[3], str[4], str[5]);
}




//consultation du cache ARP
//
int arp_cache_lookup(char *device,unsigned long ip,struct ether_addr *ether)
{
    int fd;
    struct arpreq ar;
    struct sockaddr_in *sin;

    memset((char *)&ar,0,sizeof(ar));
    strncpy(ar.arp_dev,device,sizeof(ar.arp_dev));
    sin = (struct sockaddr_in *)&ar.arp_pa;
    sin->sin_family = AF_INET;
    sin->sin_addr.s_addr = ip;
    if((fd=socket(AF_INET,SOCK_DGRAM,0))<0)
        return -1;
    if(ioctl(fd,SIOCGARP,(caddr_t)&ar)==-1)
    {
        close(fd);
        return -2;
    }
    close(fd);
    memcpy(ether->ether_addr_octet,ar.arp_ha.sa_data,ETHER_ADDR_LEN);
    return 0;
}


//affichage des erreurs
//
void die(const char* msg)
{
   fprintf(stderr,"%s\n",msg);
   exit(1);
}
